@@ -1,4 +1,4 @@
-/* $Id: log.h,v 1.7 1999/12/27 12:37:28 jurekb Exp $ */
+/* $Id: log.h,v 1.2 2000/04/17 19:38:03 jurekb Exp $ */
 /*
  *  Solid POP3 - a POP3 server
  *  Copyright (C) 1999  Jerzy Balamut <jurekb@dione.ids.pl>
@@ -31,4 +31,7 @@ void pop_log_dbg(int priority, const char *format, ...);
 void pop_error(const char *name);
 void pop_error_dbg(const char *name);
 
+extern int pop_priority;
+int check_logpriority(void *name);
+
 #endif /* log.h */
