@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: mailbox.c,v 1.39 2000/01/10 19:39:03 jurekb Exp $";
+static const char rcsid[] = "$Id: mailbox.c,v 1.6 2000/05/13 13:25:52 jurekb Exp $";
 /*
  *  Solid POP3 - a POP3 server
  *  Copyright (C) 1999  Jerzy Balamut <jurekb@dione.ids.pl>
@@ -98,7 +98,7 @@ int unlock_mailbox(void) {
 	mailunlock();
 #endif
 	return retcode;
-};
+}
 
 int lock_mailbox(void) {
 #if defined(F_SETLK) && defined(F_SETLKW) && !defined(HAVE_FLOCK)
@@ -123,14 +123,14 @@ int lock_mailbox(void) {
 	return lockf(mailboxfd, F_LOCK, 0);
 #endif /* defined(F_SETLK) && defined(F_SETLKW) */
 #endif /* HAVE_FLOCK */
-};
+}
 
 long int mb_dec(char *in) {	
 	long int value;
 	
 	value = strtol(in, NULL, 10);
 	return (((value == LONG_MIN) || (value == LONG_MAX)) ? -1 : value);
-};
+}
 
 time_t mb_readtime(char *in) {
 	int m;
@@ -162,13 +162,29 @@ time_t mb_readtime(char *in) {
 	vtm.tm_year = mb_dec(in += 3) - 1900;
 	vtm.tm_isdst = -1;
 	return mktime(&vtm);
-};
+}
+
+int mb_compare(const char *a, const char *b, int alength) {
+	if (alength < strlen(b))
+		return 0;
+	return (strncmp(a, b, strlen(b)) == 0) ? 1 : 0;
+}
+
+int mb_fixed(const char *what, int alength) {
+	return (mb_compare(what, "Received:", alength) ||
+		mb_compare(what, "Date:", alength) ||
+		mb_compare(what, "Message-Id:", alength) ||
+		mb_compare(what, "Subject:", alength));
+}
+
 
 #ifdef BULLETINS
 
 void mb_end_of_adding(void) {
 	struct stat stbuf;
 
+	if (mailboxfd < 0)
+		return;
 	if (fstat(mailboxfd, &stbuf) < 0) {
 		unlock_mailbox();
 		pop_error("mailbox: fstat");
@@ -182,16 +198,23 @@ void mb_end_of_adding(void) {
 	};
 	mailboxmtime = stbuf.st_mtime;
 	lastmailboxsize = mailboxsize = stbuf.st_size;
-};
+}
 
-void mb_add_message(int fd) {
+
+int mb_add_message(int fd) {
 	off_t tmpwhere;
 	ssize_t mcount, tmp;
 	char linebuf[128];
 	char msgdate[21];
 	struct mb_message *mbspecific;
 	struct md5_ctx context;
+	int header = 1, fixed = 0, newline = 1;
 	
+	if (mailboxfd < 0) {
+		pop_log(pop_priority, "mailbox: can't add bulletin to mailbox - \
+mailbox doesn't exist");
+		return 0;
+	}
 	if ((tmpwhere = lseek(mailboxfd, 0, SEEK_END)) < 0) {
 		unlock_mailbox();
 		close(fd);
@@ -203,7 +226,7 @@ void mb_add_message(int fd) {
 	if ((mcount = fd_fgets(linebuf, sizeof(linebuf), fd)) < 0) {
 		unlock_mailbox();
 		close(fd);
-		pop_log(POP_PRIORITY, "mailbox: can't read message");
+		pop_log(pop_priority, "mailbox: can't read message");
 		pop_error("mailbox: read");
 		send_error("fatal error");
 		exit(1);	
@@ -211,21 +234,21 @@ void mb_add_message(int fd) {
 	if (mcount < (5 + 2 + sizeof(msgdate))) { /* "From " + username + DATE */
 		unlock_mailbox();
 		close(fd);
-		pop_log(POP_PRIORITY, "mailbox: message is too short");
+		pop_log(pop_priority, "mailbox: message is too short");
 		send_error("fatal error");
 		exit(1);
 	};
 	if (strncmp(linebuf, "From ", 5) != 0) {
 		unlock_mailbox();
 		close(fd);
-		pop_log(POP_PRIORITY, "mailbox: missing \"From \"");
+		pop_log(pop_priority, "mailbox: missing \"From \"");
 		send_error("fatal error");
 		exit(1);
 	};
 	if (md_realloc(sizeof(struct mb_message)) < 0) {
 		unlock_mailbox();
 		close(fd);
-		pop_log(POP_PRIORITY, "mailbox: no memory available");
+		pop_log(pop_priority, "mailbox: no memory available");
 		send_error("no memory available");
 		exit(1);
 	};
@@ -234,20 +257,19 @@ void mb_add_message(int fd) {
 	mbspecific = ((struct mb_message *)(messages[msgnr - 1].md_specific));
 	mbspecific->from_where = tmpwhere;
 	mbspecific->from_size = 0;
-	md5_init_ctx(&context);
 	while (mcount > 0) {		
 		if (mcount < (sizeof(msgdate))) {
 			memmove(msgdate, msgdate + mcount, (sizeof(msgdate) - mcount));
 			memcpy(msgdate + (sizeof(msgdate) - mcount), linebuf, mcount);
 		} else
 			memcpy(msgdate, linebuf + mcount - sizeof(msgdate), sizeof(msgdate));
-		md5_process_bytes(linebuf, mcount, &context);
+
 		tmpwhere += mcount;
 		mbspecific->from_size += mcount;
 		if (write(mailboxfd, linebuf, mcount) != mcount) {
 			unlock_mailbox();
 			close(fd);
-			pop_log(POP_PRIORITY, "mailbox: can't write to mailbox: %.1024s", maildrop_name);
+			pop_log(pop_priority, "mailbox: can't write to mailbox: %.1024s", maildrop_name);
 			pop_error("mailbox: write");
 			send_error("fatal error");
 			exit(1);
@@ -259,7 +281,7 @@ void mb_add_message(int fd) {
 	if (mcount < 0) {
 		unlock_mailbox();
 		close(fd);
-		pop_log(POP_PRIORITY, "mailbox: can't read from message file");
+		pop_log(pop_priority, "mailbox: can't read from message file");
 		pop_error("mailbox: read");
 		send_error("message file is damaged");
 		exit(1);
@@ -267,45 +289,49 @@ void mb_add_message(int fd) {
 	if (msgdate[sizeof(msgdate) - 1] != '\n') {
 		unlock_mailbox();
 		close(fd);
-		pop_log(POP_PRIORITY, "mailbox: message file is damaged");
+		pop_log(pop_priority, "mailbox: message file is damaged");
 		send_error("can't read from message file");
 		exit(1);
 	};
 	msgdate[sizeof(msgdate) - 1] = 0;
-	if ((messages[msgnr - 1].msg_time = mb_readtime(msgdate)) == -1) {
-		unlock_mailbox();
-		close(fd);
-		pop_log(POP_PRIORITY, "mailbox: message file is damaged");
-		send_error("message file is damaged");
-		exit(1);
-	};
-	md5_finish_ctx(&context, mbspecific->digest);
+	messages[msgnr - 1].msg_time = mb_readtime(msgdate);
 	mbspecific->where = tmpwhere;	
+	md5_init_ctx(&context);
 	while ((mcount = fd_fgets(linebuf, sizeof(linebuf), fd)) > 0) {
 		if (write(mailboxfd, linebuf, mcount) != mcount) {
 			unlock_mailbox();
 			close(fd);
-			pop_log(POP_PRIORITY, "mailbox: can't write to mailbox: %.1024s", maildrop_name);
+			pop_log(pop_priority, "mailbox: can't write to mailbox: %.1024s", maildrop_name);
 			pop_error("mailbox: write");
 			send_error("fatal error");
 			exit(1);
 		};
+		if (header && newline)
+			fixed = mb_fixed(linebuf, mcount);
+		if (fixed)
+			md5_process_bytes(linebuf, mcount, &context);
 		messages[msgnr - 1].crlfsize += mcount;
 		messages[msgnr - 1].size += mcount;
 		mbspecific->from_size += mcount;
-		if (linebuf[mcount - 1] == '\n')
+		if (linebuf[mcount - 1] == '\n') {
 			messages[msgnr - 1].crlfsize++;
+			if ((mcount == 1) && newline)
+				header = 0;
+			newline = 1;
+		}
 		tmp = mcount;
 	};
+	md5_finish_ctx(&context, mbspecific->digest);
 	close(fd);
 	if (mcount < 0) {
 		unlock_mailbox();
-		pop_log(POP_PRIORITY, "mailbox: can't read from message file");
+		pop_log(pop_priority, "mailbox: can't read from message file");
 		pop_error("mailbox: read");
 		send_error("can't read from message file");
 		exit(1);
 	};
-};
+	return 1;
+}
 #endif
 
 
@@ -322,7 +348,7 @@ int mb_parse(int compare) {
 	off_t tmpfrom_where = 0, tmpwhere = 0;
 	size_t tmpfrom_size = 0, tmpsize = 0, tmpcrlfsize = 0;
 	char tmpdigest[16];
-	int newline = 1;
+	int newline = 1, header, fixed;
 	
 	if (compare)
 		if (lseek(mailboxfd, 0, SEEK_SET) < 0) {
@@ -334,7 +360,7 @@ int mb_parse(int compare) {
 		if (md_alloc(sizeof(struct mb_message)) < 0) {
 			unlock_mailbox();
 			mb_release();
-			pop_log(POP_PRIORITY, "mailbox: no memory available");
+			pop_log(pop_priority, "mailbox: no memory available");
 			send_error("no memory available");
 			exit(1);
 		};
@@ -344,7 +370,7 @@ int mb_parse(int compare) {
 			unlock_mailbox();
 			mb_release();
 		};
-		pop_log(POP_PRIORITY, "mailbox: mailbox %.1024s is damaged", maildrop_name);
+		pop_log(pop_priority, "mailbox: mailbox %.1024s is damaged", maildrop_name);
 		pop_error("mailbox: read");
 		if (!compare) {
 			send_error("mailbox is damaged");
@@ -364,7 +390,7 @@ int mb_parse(int compare) {
 			unlock_mailbox();
 			mb_release();
 		};
-		pop_log(POP_PRIORITY, "mailbox: mailbox %.1024s is damaged", maildrop_name);
+		pop_log(pop_priority, "mailbox: mailbox %.1024s is damaged", maildrop_name);
 		if (!compare) {
 			send_error("mailbox is damaged");
 			exit(1);
@@ -376,7 +402,7 @@ int mb_parse(int compare) {
 			unlock_mailbox();
 			mb_release();
 		};
-		pop_log(POP_PRIORITY, "mailbox: mailbox %.1024s is damaged", maildrop_name);
+		pop_log(pop_priority, "mailbox: mailbox %.1024s is damaged", maildrop_name);
 		if (!compare) {
 			send_error("mailbox is damaged");
 			exit(1);
@@ -387,11 +413,13 @@ int mb_parse(int compare) {
 	while (mcount > 0) {
 		act_ofs += mcount;
 		if ((newline) && (mcount >= (5 + 2 + sizeof(msgdate))) && (strncmp(mbuf, "From ", 5) == 0)) {
+			header = 1; fixed = 0;
 			if (!compare)
 				messages[tmpmsgnr].deleted = messages[tmpmsgnr].cread = 0;
 			if (tmpmsgnr > 0) {
-				if (!compare) {
 					tmp = (struct mb_message *)(messages[tmpmsgnr - 1].md_specific);
+					md5_finish_ctx(&context, tmpdigest);
+				if (!compare) {
 					tmp->from_where = tmpfrom_where;
 					messages[tmpmsgnr - 1].size = tmpsize;
 					messages[tmpmsgnr - 1].crlfsize = tmpcrlfsize;
@@ -400,7 +428,6 @@ int mb_parse(int compare) {
 					memcpy(tmp->digest, tmpdigest, 16);
 					messages[tmpmsgnr - 1].msg_time = tmpmsg_time;
 				} else {
-					tmp = (struct mb_message *)(messages[tmpmsgnr - 1].md_specific);
 					if ((tmp->from_where != tmpfrom_where) ||
 					    (messages[tmpmsgnr - 1].size != tmpsize) ||
 					    (messages[tmpmsgnr - 1].crlfsize != tmpcrlfsize) ||
@@ -421,12 +448,11 @@ int mb_parse(int compare) {
 				if (md_realloc(sizeof(struct mb_message)) < 0) {
 					unlock_mailbox();
 					mb_release();
-					pop_log(POP_PRIORITY, "mailbox: no memory available");
+					pop_log(pop_priority, "mailbox: no memory available");
 					send_error("no memory available");
 					exit(1);
 				};
 			md5_init_ctx(&context);
-			md5_process_bytes(mbuf, mcount, &context);
 			tmpfrom_size = mcount;
 			if (mcount < (sizeof(msgdate))) {
 				memmove(msgdate, msgdate + mcount, (sizeof(msgdate) - mcount));
@@ -444,17 +470,17 @@ int mb_parse(int compare) {
 					memcpy(msgdate, mbuf + mcount - sizeof(msgdate), sizeof(msgdate));
 				act_ofs += mcount;
 				tmpfrom_size += mcount;
-				md5_process_bytes(mbuf, mcount, &context);
+				
 			};
-			md5_finish_ctx(&context, tmpdigest);
+			
 			msgdate[sizeof(msgdate) - 1] = 0;
 			tmpmsg_time = mb_readtime(msgdate);
-			if ((mcount <= 0) || (tmpmsg_time == -1)) {
+			if (mcount <= 0) {
 				if (!compare) {
 					unlock_mailbox();
 					mb_release();
 				};
-				pop_log(POP_PRIORITY, "mailbox: mailbox is damaged: %.1024s", maildrop_name);
+				pop_log(pop_priority, "mailbox: mailbox is damaged: %.1024s", maildrop_name);
 				if (!compare) {
 					send_error("mailbox is damaged");
 					exit(1);
@@ -468,12 +494,18 @@ int mb_parse(int compare) {
 			tmpsize += mcount;
 			tmpcrlfsize += mcount;
 			tmpfrom_size += mcount;
+			if (header && newline)
+				fixed = mb_fixed(mbuf, mcount);
+			if (fixed)
+				md5_process_bytes(mbuf, mcount, &context);
 			if (mbuf[mcount - 1] == '\n') {
 				tmpcrlfsize++;
+				if (newline && (mcount == 1))
+					header = 0;
 				newline = 1;
 			} else
 				newline = 0;
-		};		
+		};
 		tmp2 = mcount;
 		mcount = fd_fgets(mbuf, sizeof(mbuf), mailboxfd);		
 	};
@@ -482,7 +514,7 @@ int mb_parse(int compare) {
 			unlock_mailbox();
 			mb_release();
 		};
-		pop_log(POP_PRIORITY, "mailbox: mailbox is damaged: %.1024s", maildrop_name);
+		pop_log(pop_priority, "mailbox: mailbox is damaged: %.1024s", maildrop_name);
 		pop_error("mailbox: read");
 		if (!compare) {
 			send_error("mailbox is damaged");
@@ -496,7 +528,7 @@ int mb_parse(int compare) {
 				unlock_mailbox();
 				mb_release();
 			};
-			pop_log(POP_PRIORITY, "mailbox: mailbox is damaged: %.1024s", maildrop_name);			
+			pop_log(pop_priority, "mailbox: mailbox is damaged: %.1024s", maildrop_name);			
 			if (!compare) {
 				send_error("mailbox is damaged");
 				exit(1);
@@ -504,6 +536,7 @@ int mb_parse(int compare) {
 			return -1;
 		};
 	tmp = (struct mb_message *)(messages[tmpmsgnr - 1].md_specific);
+	md5_finish_ctx(&context, tmpdigest);
 	if (!compare) {	
 		messages[tmpmsgnr - 1].deleted = messages[tmpmsgnr - 1].cread = 0;
 		tmp->from_where = tmpfrom_where;
@@ -525,73 +558,90 @@ int mb_parse(int compare) {
 			    return -1;		
 	};	
 	return 0;
-};
+}
 
 void mb_init(void) {	
 	struct stat stbuf;
 	
+	msgnr = 0;
 	if ((mailboxfd = open(maildrop_name, O_RDWR)) < 0) {
 #ifdef CREATEMAILDROP
 		if ((errno == ENOENT) && (createmaildrop))
 			mailboxfd = open(maildrop_name, O_RDWR | O_CREAT | O_EXCL, 0600);
-		if (mailboxfd < 0) {
-#endif
-		pop_log(POP_PRIORITY, "mailbox: can't open mailbox file: %.1024s", maildrop_name);
-		pop_error("mailbox: open");
-		send_error("can't open mailbox file");
-		exit(1);
-#ifdef CREATEMAILDROP
+		if ((mailboxfd < 0) && (createmaildrop)) {
+			pop_log(pop_priority, "mailbox: can't open mailbox file: %.1024s", maildrop_name);
+			pop_error("mailbox: open");
+			send_error("can't open mailbox file");
+			exit(1);
 		};
 #endif
+		if ((mailboxfd < 0) && (errno != ENOENT)) {
+			pop_log(pop_priority, "mailbox: can't open mailbox file: %.1024s", maildrop_name);
+			pop_error("mailbox: open");
+			send_error("can't open mailbox file");
+			exit(1);
+		};
 	};
-	if (lock_mailbox() < 0) {
-		pop_log(POP_PRIORITY, "mailbox: can't set lock on mailbox file: %.1024s", maildrop_name);
-		pop_error(MLNAME);
-		send_error("can't lock mailbox file");
-		exit(1);
-	};
-	if (fstat(mailboxfd, &stbuf) < 0) {
-		unlock_mailbox();
-		pop_error("mailbox: fstat");
-		send_error("fstat failed");
-		exit(1);
-	};
-	if (!S_ISREG(stbuf.st_mode)) {
-		unlock_mailbox();
-		pop_log(POP_PRIORITY, "mailbox: mailbox is not regular file: %.1024s", maildrop_name);
-		send_error("mailbox is not regular file");
-		exit(1);
-	};
+	if (mailboxfd > 0) {
+		if (lock_mailbox() < 0) {
+			pop_log(pop_priority, "mailbox: can't set lock on mailbox file: %.1024s", maildrop_name);
+			pop_error(MLNAME);
+			send_error("can't lock mailbox file");
+			exit(1);
+		};
+		if (fstat(mailboxfd, &stbuf) < 0) {
+			unlock_mailbox();
+			pop_error("mailbox: fstat");
+			send_error("fstat failed");
+			exit(1);
+		};
+		if (!S_ISREG(stbuf.st_mode)) {
+			unlock_mailbox();
+			pop_log(pop_priority, "mailbox: mailbox is not regular file: %.1024s", maildrop_name);
+			send_error("mailbox is not regular file");
+			exit(1);
+		};
+	}
 #ifndef BULLETINS
 	mailboxmtime = stbuf.st_mtime;
 	lastmailboxsize = mailboxsize = stbuf.st_size;
 #endif
-	mb_parse(0);
+	if (mailboxfd > 0)
+		mb_parse(0);
 #ifndef BULLETINS
-	if (unlock_mailbox() < 0) {
-		pop_log(POP_PRIORITY, "mailbox: can't remove lock from mailbox file: %.1024s", maildrop_name);
-		pop_error(MLNAME);
-		send_error("can't unlock mailbox file");
-		exit(1);
-	};
+	if (mailboxfd > 0)
+		if (unlock_mailbox() < 0) {
+			pop_log(pop_priority, "mailbox: can't remove lock from mailbox file: %.1024s", maildrop_name);
+			pop_error(MLNAME);
+			send_error("can't unlock mailbox file");
+			exit(1);
+		};
 #endif
-};
+}
 
 void mb_release(void)
 {
-	close(mailboxfd);
+	if (mailboxfd > 0)
+		close(mailboxfd);
 	md_free();
-};
+}
 
-void mb_reparse(void) { // mailbox must be locked
+void mb_reparse(void) {
 	struct stat stbuf;
 	
+	if (mailboxfd < 0)
+		return;
 	if (fstat(mailboxfd, &stbuf) < 0) {
 		pop_error("mailbox: fstat");
 		exit(1);
 	};
 	if ((stbuf.st_size != lastmailboxsize) ||
 	    (stbuf.st_mtime != mailboxmtime)) {
+		if (lock_mailbox() < 0) {
+			pop_log(pop_priority, "mailbox: can't set lock on mailbox file: %.1024s", maildrop_name);
+			pop_error(MLNAME);
+			exit(1);
+		};
 		if (fstat(mailboxfd, &stbuf) < 0) {
 			unlock_mailbox();
 			pop_error("mailbox: fstat");
@@ -599,22 +649,28 @@ void mb_reparse(void) { // mailbox must be locked
 		};
 		if (stbuf.st_size < mailboxsize) {
 			unlock_mailbox();
-			pop_log(POP_PRIORITY, "mailbox: mailbox content has been changed");
+			pop_log(pop_priority, "mailbox: mailbox content has been changed");
 			exit(1);
 		};				
 		if (mb_parse(1) < 0) {
 			unlock_mailbox();
-			pop_log(POP_PRIORITY, "mailbox: mailbox content has been changed");
+			pop_log(pop_priority, "mailbox: mailbox content has been changed");
 			exit(1);
 		};
 		lastmailboxsize = stbuf.st_size;
 		mailboxmtime = stbuf.st_mtime;
+		if (unlock_mailbox() < 0) {
+			pop_log(pop_priority, "mailbox: can't remove lock from mailbox file: %.1024s", maildrop_name);
+			pop_error(MLNAME);
+			exit(1);
+		};
 	};
-};
+}
 
 void mb_cleanup(void) {
-	unlock_mailbox();
-};
+	if (mailboxfd > 0)
+		unlock_mailbox();
+}
 
 void mb_retrieve(unsigned int nr) {
 	if (nr > msgnr) {
@@ -627,11 +683,6 @@ void mb_retrieve(unsigned int nr) {
 		check_wccount();
 		return;
 	};
-	if (lock_mailbox() < 0) {
-		pop_log(POP_PRIORITY, "mailbox: can't set lock on mailbox file: %.1024s", maildrop_name);
-		pop_error(MLNAME);
-		exit(1);
-	};
 	mb_reparse();
 	if (lseek(mailboxfd, ((struct mb_message *) (messages[nr].md_specific))->where, SEEK_SET) < 0) {
 		unlock_mailbox();
@@ -639,12 +690,9 @@ void mb_retrieve(unsigned int nr) {
 		exit(1);
 	};
 	md_retrieve(nr, mailboxfd, &mb_cleanup);
-	if (unlock_mailbox() < 0) {
-		pop_log(POP_PRIORITY, "mailbox: can't remove lock from mailbox file: %.1024s", maildrop_name);
-		pop_error(MLNAME);
-		exit(1);
-	};
-};
+	mb_reparse();
+	md_end_reply(&mb_cleanup);
+}
 
 void mb_copy(off_t ofs, size_t length) {
 	char mb_buf[10240];
@@ -701,15 +749,17 @@ void mb_copy(off_t ofs, size_t length) {
 		};
 		curofs += tmp;
 	};
-};
+}
 
 void mb_update(void) {
 	unsigned int tmp;
 	off_t newsize;
 	struct stat stbuf;
 	
+	if (mailboxfd < 0)
+		return;
 	if (lock_mailbox() < 0) {
-		pop_log(POP_PRIORITY, "mailbox: can't set lock on mailbox: %.1024s", maildrop_name);
+		pop_log(pop_priority, "mailbox: can't set lock on mailbox: %.1024s", maildrop_name);
 		pop_error(MLNAME);
 		send_error("can't set lock on mailbox");
 		exit(1);
@@ -724,13 +774,13 @@ void mb_update(void) {
 	    (stbuf.st_mtime != mailboxmtime)) {
 		if (stbuf.st_size < mailboxsize) {
 			unlock_mailbox();
-			pop_log(POP_PRIORITY, "mailbox: mailbox content has been changed: %.1024s", maildrop_name);
+			pop_log(pop_priority, "mailbox: mailbox content has been changed: %.1024s", maildrop_name);
 			send_error("fatal error");
 			exit(1);
 		};
 		if (mb_parse(1) < 0) {
 			unlock_mailbox();
-			pop_log(POP_PRIORITY, "mailbox: mailbox content has been changed: %.1024s", maildrop_name);
+			pop_log(pop_priority, "mailbox: mailbox content has been changed: %.1024s", maildrop_name);
 			send_error("fatal error");
 			exit(1);
 		};
@@ -770,18 +820,18 @@ void mb_update(void) {
 			exit(1);
 		};
 	if (unlock_mailbox() < 0) {
-		pop_log(POP_PRIORITY, "mailbox: can't remove lock from mailbox: %.1024s", maildrop_name);
+		pop_log(pop_priority, "mailbox: can't remove lock from mailbox: %.1024s", maildrop_name);
 		pop_error(MLNAME);
 		send_error("can't remove lock from mailbox");
 		exit(1);
 	};
 	if (close(mailboxfd) < 0) {
-		pop_log(POP_PRIORITY, "mailbox: can't close mailbox: %.1024s", maildrop_name);
+		pop_log(pop_priority, "mailbox: can't close mailbox: %.1024s", maildrop_name);
 		pop_error("mailbox: close");
 		send_error("can't close mailbox");
 		exit(1);
 	};
-};
+}
 
 void mb_top(unsigned int nr, unsigned int lcount) {
 	if (nr > msgnr) {
@@ -794,12 +844,6 @@ void mb_top(unsigned int nr, unsigned int lcount) {
 		check_wccount();
 		return;
 	};
-	if (lock_mailbox() < 0) {
-		pop_log(POP_PRIORITY, "mailbox: can't set lock on mailbox file: %.1024s", maildrop_name);
-		pop_error(MLNAME);
-		exit(1);
-	};
-
 	mb_reparse();
 	if (lseek(mailboxfd, ((struct mb_message *) (messages[nr].md_specific))->where, SEEK_SET) < 0) {
 		unlock_mailbox();
@@ -807,14 +851,11 @@ void mb_top(unsigned int nr, unsigned int lcount) {
 		exit(1);
 	};
 	md_top(nr, lcount, mailboxfd, &mb_cleanup);
-	if (unlock_mailbox() < 0) {
-		pop_log(POP_PRIORITY, "mailbox: can't remove lock from mailbox file: %.1024s", maildrop_name);
-		pop_error(MLNAME);
-		exit(1);
-	};
-};
+	mb_reparse();
+	md_end_reply(&mb_cleanup);
+}
 
 char *mb_md5_uidl_message(unsigned int number, char *result) {
 	memcpy(result, ((struct mb_message *)(messages[number].md_specific))->digest, 16);
 	return result;
-};
+}
