@@ -1,4 +1,4 @@
-/* $Id: mailbox.h,v 1.8 1999/11/20 22:24:20 jurekb Exp $ */
+/* $Id: mailbox.h,v 1.2 2000/05/13 13:25:52 jurekb Exp $ */
 /*
  *  Solid POP3 - a POP3 server
  *  Copyright (C) 1999  Jerzy Balamut <jurekb@dione.ids.pl>
@@ -31,7 +31,7 @@ void mb_retrieve(unsigned int);
 void mb_top(unsigned int, unsigned int);
 char *mb_md5_uidl_message(unsigned int, char *);
 #ifdef BULLETINS
-void mb_add_message(int);
+int mb_add_message(int);
 void mb_end_of_adding(void);
 #endif
 
