@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: maildir.c,v 1.32 1999/12/28 22:18:36 jurekb Exp $";
+static const char rcsid[] = "$Id: maildir.c,v 1.5 2000/05/13 13:25:52 jurekb Exp $";
 /*
  *  Solid POP3 - a POP3 server
  *  Copyright (C) 1999  Jerzy Balamut <jurekb@dione.ids.pl>
@@ -66,6 +66,7 @@ extern char username[];
 extern int msgnr, maxmsgnr, msgdel;
 extern struct message *messages;
 int msgfd;
+int mdir_exists;
 #ifdef BULLETINS
 pid_t nomsg = 0, nnomsg = 0;
 #endif
@@ -99,14 +100,16 @@ int mdir_compare(const void *arg1, const void *arg2) {
         if (d1 == d2)
 		return 0;
 	return 1;
-};
+}
 
 #ifdef BULLETINS
 void mdir_end_of_adding(void) {
+	if (mdir_exists == 0)
+		return;
 	qsort(messages, msgnr, sizeof(struct message), mdir_compare);
-};
+}
 
-void mdir_add_message(int fd) {
+int mdir_add_message(int fd) {
 	char filename[PATH_MAX];
 	char nfilename[PATH_MAX];
 	char msgname[100 + 256];
@@ -118,9 +121,14 @@ void mdir_add_message(int fd) {
 	ssize_t mcount, tmp;
 	char linebuf[128];
 	
+	if (mdir_exists == 0) {
+		pop_log(pop_priority, "maildir: can't add bulletin - maildir \
+doesn't exist");
+		return 0;
+	}
 	if (gethostname(hstname, sizeof(hstname)) < 0) {
 		close(fd);
-		pop_log(POP_PRIORITY, "maildir: can't get name of host");
+		pop_log(pop_priority, "maildir: can't get name of host");
 		pop_error("maildir: gethostname");
 		send_error("fatal error");
 		exit(1);
@@ -129,7 +137,7 @@ void mdir_add_message(int fd) {
 		snprintf(msgname, sizeof(msgname), "tmp/%u.%u.%.256s", tmptime, tmppid, hstname);
 		if ((strlen(msgname) + strlen(maildrop_name) + 2) > sizeof(filename)) {
 			close(fd);
-			pop_log(POP_PRIORITY, "maildir: message file name is too long");
+			pop_log(pop_priority, "maildir: message file name is too long");
 			send_error("fatal error");
 			exit(1);
 		};
@@ -142,7 +150,7 @@ void mdir_add_message(int fd) {
 			nomsg++;
 			if (nomsg > 500) {
 				close(fd);
-				pop_log(POP_PRIORITY, "maildir: can't found unused message file name");
+				pop_log(pop_priority, "maildir: can't found unused message file name");
 				send_error("fatal error");
 				exit(1);
 			};
@@ -150,7 +158,7 @@ void mdir_add_message(int fd) {
 		};
 		if (nmsgfd < 0) {
 			close(fd);
-			pop_log(POP_PRIORITY, "maildir: can't create message file: %.1024s", filename);
+			pop_log(pop_priority, "maildir: can't create message file: %.1024s", filename);
 			pop_error("maildir: open");
 			send_error("fatal error");
 			exit(1);			
@@ -162,7 +170,7 @@ void mdir_add_message(int fd) {
 		close(nmsgfd);
 		close(fd);
 		unlink(filename);
-		pop_log(POP_PRIORITY, "maildir: can't read message");
+		pop_log(pop_priority, "maildir: can't read message");
 		pop_error("maildir: read");
 		send_error("fatal error");
 		exit(1);	
@@ -174,7 +182,7 @@ void mdir_add_message(int fd) {
 					close(nmsgfd);
 					close(fd);
 					unlink(filename);
-					pop_log(POP_PRIORITY, "maildir: can't read message");
+					pop_log(pop_priority, "maildir: can't read message");
 					pop_error("maildir: read");
 					send_error("fatal error");
 					exit(1);
@@ -186,7 +194,7 @@ void mdir_add_message(int fd) {
 				close(nmsgfd);
 				close(fd);
 				unlink(filename);
-				pop_log(POP_PRIORITY, "maildir: empty message");
+				pop_log(pop_priority, "maildir: empty message");
 				send_error("fatal error");
 				exit(1);
 			};
@@ -196,7 +204,7 @@ void mdir_add_message(int fd) {
 		close(nmsgfd);
 		close(fd);
 		unlink(filename);
-		pop_log(POP_PRIORITY, "maildir: no memory available");
+		pop_log(pop_priority, "maildir: no memory available");
 		send_error("no memory available");
 		exit(1);
 	};
@@ -210,7 +218,7 @@ void mdir_add_message(int fd) {
 			close(nmsgfd);
 			close(fd);
 			unlink(filename);
-			pop_log(POP_PRIORITY, "maildir: can't write to message file: %.1024s", filename);
+			pop_log(pop_priority, "maildir: can't write to message file: %.1024s", filename);
 			pop_error("maildir: write");
 			send_error("fatal errror");
 			exit(1);
@@ -226,7 +234,7 @@ void mdir_add_message(int fd) {
 		close(nmsgfd);
 		close(fd);
 		unlink(filename);
-		pop_log(POP_PRIORITY, "maildir: can't read from message file: %.1024s", filename);
+		pop_log(pop_priority, "maildir: can't read from message file: %.1024s", filename);
 		pop_error("maildir: read");
 		send_error("can't read from message file");
 		exit(1);
@@ -236,14 +244,14 @@ void mdir_add_message(int fd) {
 			close(nmsgfd);
 			close(fd);
 			unlink(filename);
-			pop_log(POP_PRIORITY, "maildir: message is damaged: %.1024s", filename);
+			pop_log(pop_priority, "maildir: message is damaged: %.1024s", filename);
 			send_error("message is damaged");
 			exit(1);
 		};
 	if (close(nmsgfd) < 0) {
 		close(fd);
 		unlink(filename);
-		pop_log(POP_PRIORITY, "maildir: can't close message file: %.1024s", filename);
+		pop_log(pop_priority, "maildir: can't close message file: %.1024s", filename);
 		pop_error("maildir: close");
 		send_error("can't close message file");
 		exit(1);
@@ -253,7 +261,7 @@ void mdir_add_message(int fd) {
 		if ((strlen(msgname) + strlen(maildrop_name) + 2) > sizeof(nfilename)) {
 			close(fd);
 			unlink(filename);
-			pop_log(POP_PRIORITY, "maildir: message file name is too long");
+			pop_log(pop_priority, "maildir: message file name is too long");
 			send_error("fatal error");
 			exit(1);
 		};
@@ -267,7 +275,7 @@ void mdir_add_message(int fd) {
 			if (nnomsg > 500) {
 				close(fd);
 				unlink(filename);
-				pop_log(POP_PRIORITY, "maildir: can't found unused message file name");
+				pop_log(pop_priority, "maildir: can't found unused message file name");
 				send_error("fatal error");
 				exit(1);
 			};
@@ -276,7 +284,7 @@ void mdir_add_message(int fd) {
 		if (nmsgfd < 0) {
 			close(fd);
 			unlink(filename);
-			pop_log(POP_PRIORITY, "maildir: can't link message file: %.1024s", nfilename);
+			pop_log(pop_priority, "maildir: can't link message file: %.1024s", nfilename);
 			pop_error("maildir: link");
 			send_error("fatal error");
 			exit(1);			
@@ -288,12 +296,13 @@ void mdir_add_message(int fd) {
 		close(nmsgfd);
 		close(fd);
 		unlink(nfilename); /* remove message */
-		pop_log(POP_PRIORITY, "maildir: no memory available");
+		pop_log(pop_priority, "maildir: no memory available");
 		send_error("no memory available");
 		exit(1);
 	};
 	strcpy(((struct mdir_message *) (messages[msgnr - 1].md_specific))->filename, msgname);
-};
+	return 1;
+}
 #endif
 
 void mdir_clean(void) {
@@ -326,33 +335,7 @@ void mdir_clean(void) {
 	};
 	closedir(dir);
 	
-};
-
-time_t read_msg_time(const char *name) {
-	char tmp[PATH_MAX], *tmp2;
-	long int result;
-	
-	
-	if (strlen(name) >= sizeof(tmp)) {
-			    /* no overflow, already checked in mdir_append but
-			       "szczezonego Pan Bog szczeze" */
-		mdir_release();
-		pop_log(POP_PRIORITY, "maildir: unexpected error - send info to author of program");
-		send_error("fatal error");
-		exit(1);
-	};
-	strcpy(tmp, name);
-	*(strchr(tmp, '.')) = 0;
-	result = strtol(tmp, &tmp2, 10);
-	if ((result == LONG_MIN) || (result == LONG_MAX) || (*tmp2 != 0)) {
-		mdir_release();
-		pop_log(POP_PRIORITY, "maildir: wrong message file name: %.1024s", name);
-		send_error("wrong message file name");
-		exit(1);
-	};
-	return result;
-};
-	
+}
 
 void mdir_append(const char *directory) {
 	DIR *dirstream;
@@ -363,10 +346,11 @@ void mdir_append(const char *directory) {
 	int messagefd;
 	ssize_t tmp4, mcount;
 	char *fname;
+	struct stat stbuf;
 	
 	if ((strlen(directory) + strlen(maildrop_name) + 4) > sizeof(filename)) {
 		mdir_release();
-		pop_log(POP_PRIORITY, "maildir: maildir name too long");
+		pop_log(pop_priority, "maildir: maildir name too long");
 		send_error("maildir name too long");
 		exit(1);
 	};
@@ -375,28 +359,29 @@ void mdir_append(const char *directory) {
 	fname = filename + strlen(filename);
 	strcat(filename, directory);	
 	if ((dirstream = opendir(filename)) == NULL) {
-		mdir_release();
-		pop_log(POP_PRIORITY, "maildir: can't open maildir: %.1024s", filename);
-		pop_error("maildir: opendir");
-		send_error("can't open maildir");
-		exit(1);
-	};
+#ifdef CREATEMAILDROP
+		if ((createmaildrop) || (errno != ENOENT)) {
+#else
+		if (errno != ENOENT) {
+#endif
+			mdir_release();
+			pop_log(pop_priority, "maildir: can't open maildir: %.1024s", filename);
+			pop_error("maildir: opendir");
+			send_error("can't open maildir");
+			exit(1);
+		}
+		mdir_exists = 0;
+		return;
+	}
 	strcat(filename, "/");
 	tmp = filename + strlen(filename);
 	while ((direntry = readdir(dirstream)) != NULL) {
 		if (direntry->d_name[0] == '.')
 			continue;
-		if (strchr(direntry->d_name, '.') == NULL) {
-			closedir(dirstream);
-			mdir_release();
-			pop_log(POP_PRIORITY, "maildir: wrong message file name: %.1024s", direntry->d_name);
-			send_error("wrong message file name");
-			exit(1);
-		};
 		if ((NAMLEN(direntry) + strlen(filename) + 1) > sizeof(filename)) {
 			closedir(dirstream);
 			mdir_release();
-			pop_log(POP_PRIORITY, "maildir: message file name too long");
+			pop_log(pop_priority, "maildir: message file name too long");
 			send_error("message file name too long");
 			exit(1);
 		};
@@ -404,30 +389,38 @@ void mdir_append(const char *directory) {
 		if (md_realloc(sizeof(struct mdir_message)) < 0) {
 			closedir(dirstream);
 			mdir_release();
-			pop_log(POP_PRIORITY, "maildir: no memory available");
+			pop_log(pop_priority, "maildir: no memory available");
 			send_error("no memory available");
 			exit(1);
 		};
 		if ((((struct mdir_message *) (messages[msgnr].md_specific))->filename = (char *) malloc(strlen(fname) + 1)) == NULL) {
 			closedir(dirstream);
 			mdir_release();
-			pop_log(POP_PRIORITY, "maildir: no memory available");
+			pop_log(pop_priority, "maildir: no memory available");
 			send_error("no memory available");
 			exit(1);
 		};
 		strcpy(((struct mdir_message *) (messages[msgnr].md_specific))->filename, fname);
-		messages[msgnr].msg_time = read_msg_time(tmp);
 		messages[msgnr].deleted = messages[msgnr].crlfsize =\
 		messages[msgnr].size = messages[msgnr].cread = 0;
-		msgnr++;
 		if ((messagefd = open(filename, O_RDONLY)) < 0) {
 			closedir(dirstream);
 			mdir_release();
-			pop_log(POP_PRIORITY, "maildir: can't open message file: %.1024s", filename);
+			pop_log(pop_priority, "maildir: can't open message file: %.1024s", filename);
 			pop_error("maildir: open");
 			send_error("can't open message file");
 			exit(1);
 		};
+		if (fstat(messagefd, &stbuf) < 0) {
+			closedir(dirstream);
+			mdir_release();
+			pop_log(pop_priority, "maildir: fstat failed: %.1024s", filename);
+			pop_error("maildir: fstat");
+			send_error("fatal error");
+			exit(1);
+		};
+		messages[msgnr].msg_time = stbuf.st_mtime;
+		msgnr++;
 		*tmp = 0;
 		fd_initfgets();
 		tmp4 = 0;
@@ -442,7 +435,7 @@ void mdir_append(const char *directory) {
 			close(messagefd);
 			closedir(dirstream);
 			mdir_release();
-			pop_log(POP_PRIORITY, "maildir: can't read from message file: %.1024s", filename);
+			pop_log(pop_priority, "maildir: can't read from message file: %.1024s", filename);
 			pop_error("maildir: read");
 			send_error("can't read from message file");
 			exit(1);
@@ -452,31 +445,31 @@ void mdir_append(const char *directory) {
 				close(messagefd);
 				closedir(dirstream);
 				mdir_release();
-				pop_log(POP_PRIORITY, "maildir: message is damaged: %.1024s", filename);
+				pop_log(pop_priority, "maildir: message is damaged: %.1024s", filename);
 				send_error("message is damaged");
 				exit(1);
 			};
 		if (close(messagefd) < 0) {
 			closedir(dirstream);
 			mdir_release();
-			pop_log(POP_PRIORITY, "maildir: can't close message file: %.1024s", filename);
+			pop_log(pop_priority, "maildir: can't close message file: %.1024s", filename);
 			pop_error("maildir: close");
 			send_error("can't close message file");
 			exit(1);
 		};
 	};
 	closedir(dirstream);
-};
+}
 
 #ifdef CREATEMAILDROP
 void mdir_mkdir(char *dirname) {
 	if (mkdir(dirname, 0700) < 0) {
-		pop_log(POP_PRIORITY, "maildir: can't create directory: %.1024s", dirname);
+		pop_log(pop_priority, "maildir: can't create directory: %.1024s", dirname);
 		pop_error("maildir: mkdir");
 		send_error("fatal error");
 		exit(1);
 	};
-};
+}
 
 void mdir_create(void) {
 	char mdrop[PATH_MAX], *tmp;
@@ -484,7 +477,7 @@ void mdir_create(void) {
 	
 	if ((strlen(maildrop_name) + 5) > sizeof(mdrop)) {
 		mdir_release();
-		pop_log(POP_PRIORITY, "maildir: maildrop name too long");
+		pop_log(pop_priority, "maildir: maildrop name too long");
 		send_error("fatal error");
 		exit(1);
 	};
@@ -499,16 +492,18 @@ void mdir_create(void) {
 	mdir_mkdir(mdrop);
 	strcpy(tmp, "/tmp");
 	mdir_mkdir(mdrop);
-};
+}
 #endif
 
 
 void mdir_init()
 {
+	mdir_exists = 1;
+	msgnr = 0;
 	mdir_clean();	
 	if (md_alloc(sizeof(struct mdir_message)) < 0) {		
 		mdir_release();
-		pop_log(POP_PRIORITY, "maildir: no memory available");
+		pop_log(pop_priority, "maildir: no memory available");
 		send_error("no memory available");
 		exit(1);
 	};
@@ -517,11 +512,13 @@ void mdir_init()
 		mdir_create();
 #endif
 	mdir_append("new");
-	mdir_append("cur");
+	if (mdir_exists)
+		mdir_append("cur");
 #ifndef BULLETINS
-	qsort(messages, msgnr, sizeof(struct message), mdir_compare);
+	if (mdir_exists)
+		qsort(messages, msgnr, sizeof(struct message), mdir_compare);
 #endif
-};
+}
 
 void mdir_release()
 {
@@ -532,11 +529,11 @@ void mdir_release()
 	for (number = 0; number < msgnr; number++)
 		free(((struct mdir_message *) (messages[number].md_specific))->filename);
 	md_free();
-};
+}
 
 void mdir_cleanup(void) {
 	close(msgfd);
-};
+}
 
 void mdir_retrieve(unsigned int nr)
 {
@@ -555,31 +552,33 @@ void mdir_retrieve(unsigned int nr)
 	if ((strlen(maildrop_name) + strlen(((struct mdir_message *)(messages[nr].md_specific))->filename)
 	     + 2) > sizeof(msgfile)) { /* it was alredy checked in md_append, 
 	                            	  but "szczezonego Pan Bog szczeze" */
-		pop_log(POP_PRIORITY, "maildir: unexpected error");
+		pop_log(pop_priority, "maildir: unexpected error");
 		exit(1);
 	};
 	strcpy(msgfile, maildrop_name);
 	strcat(msgfile, "/");
 	strcat(msgfile, ((struct mdir_message *)(messages[nr].md_specific))->filename);
 	if ((msgfd = open(msgfile, O_RDONLY)) < 0) {
-		pop_log(POP_PRIORITY, "maildir: maildir content probably has been changed");
+		pop_log(pop_priority, "maildir: maildir content probably has been changed");
 		pop_error("maildir: open");
 		exit(1);
 	};
 	md_retrieve(nr, msgfd, &mdir_cleanup);
+	md_end_reply(&mdir_cleanup);
 	close(msgfd);
-};
+}
 
 void mdir_update() {
 	int tmp;
 	char filename[PATH_MAX], newfilename[PATH_MAX], *tmp2;
 
-	
+	if (mdir_exists == 0)
+		return;
 	if ((strlen(maildrop_name) + 3) > sizeof(filename)) {
 			    /* no overflow, already checked in mdir_append but
 			       "szczezonego Pan Bog szczeze" */
 		mdir_release();
-		pop_log(POP_PRIORITY, "maildir: unexpected error - send info to author of program");
+		pop_log(pop_priority, "maildir: unexpected error - send info to author of program");
 		send_error("fatal error");
 		exit(1);
 	};
@@ -594,7 +593,7 @@ void mdir_update() {
 			    /* no overflow, already checked in mdir_append but
 			       "szczezonego Pan Bog szczeze" */
 			mdir_release();
-			pop_log(POP_PRIORITY, "maildir: unexpected error - send info to author of program");
+			pop_log(pop_priority, "maildir: unexpected error - send info to author of program");
 			send_error("fatal error");
 			exit(1);
 		};
@@ -614,7 +613,7 @@ void mdir_update() {
 		};		
 	};
 	*tmp2 = 0;
-};
+}
 
 void mdir_top(unsigned int nr, unsigned int lcount) {
 	char msgfile[PATH_MAX];
@@ -632,20 +631,21 @@ void mdir_top(unsigned int nr, unsigned int lcount) {
 	if ((strlen(maildrop_name) + strlen(((struct mdir_message *)(messages[nr].md_specific))->filename)
 	     + 2) > sizeof(msgfile)) { /* it was alredy checked in md_append, 
 	                            	  but "szczezonego Pan Bog szczeze" */
-		pop_log(POP_PRIORITY, "maildir: unexpected error");
+		pop_log(pop_priority, "maildir: unexpected error");
 		exit(1);
 	};
 	strcpy(msgfile, maildrop_name);
 	strcat(msgfile, "/");
 	strcat(msgfile, ((struct mdir_message *)(messages[nr].md_specific))->filename);
 	if ((msgfd = open(msgfile, O_RDONLY)) < 0) {
-		pop_log(POP_PRIORITY, "maildir: maildir content probably has been changed");
+		pop_log(pop_priority, "maildir: maildir content probably has been changed");
 		pop_error("maildir: open");
 		exit(1);
 	};
 	md_top(nr, lcount, msgfd, &mdir_cleanup);
+	md_end_reply(&mdir_cleanup);
 	close(msgfd);
-};
+}
 
 char *mdir_md5_uidl_message(unsigned int number, char *result) {
 	size_t length;
@@ -664,4 +664,4 @@ char *mdir_md5_uidl_message(unsigned int number, char *result) {
 	md5_process_bytes(tmp3, length, &context);
 	md5_finish_ctx(&context, result);
 	return result;
-};
+}
