@@ -1,4 +1,4 @@
-/* $Id: maildir.h,v 1.8 1999/11/20 22:24:20 jurekb Exp $ */
+/* $Id: maildir.h,v 1.3 2000/05/13 13:25:52 jurekb Exp $ */
 /*
  *  Solid POP3 - a POP3 server
  *  Copyright (C) 1999  Jerzy Balamut <jurekb@dione.ids.pl>
@@ -30,9 +30,9 @@ void mdir_retrieve(unsigned int);
 void mdir_top(unsigned int, unsigned int);
 char *mdir_md5_uidl_message(unsigned int, char *);
 #ifdef BULLETINS
-void mdir_add_message(int fd);
+int mdir_add_message(int fd);
 void mdir_end_of_adding(void);
-#endif;
+#endif
 
 struct mdir_message {
 	char *filename;
