@@ -1,4 +1,4 @@
-static const char rcsid[] = "$Id: main.c,v 1.67 2000/01/27 12:02:12 jurekb Exp $";
+static const char rcsid[] = "$Id: main.c,v 1.8 2000/05/13 13:25:52 jurekb Exp $";
 /*
  *  Solid POP3 - a POP3 server
  *  Copyright (C) 1999  Jerzy Balamut <jurekb@dione.ids.pl>
@@ -35,6 +35,7 @@ static const char rcsid[] = "$Id: main.c,v 1.67 2000/01/27 12:02:12 jurekb Exp $
 #include <sys/socket.h>
 #include <sys/stat.h>
 #include <fcntl.h>
+#include <errno.h>
 #include <netinet/in.h>
 #include <signal.h>
 #ifdef HAVE_GRP_H
@@ -66,6 +67,7 @@ static const char rcsid[] = "$Id: main.c,v 1.67 2000/01/27 12:02:12 jurekb Exp $
 #include <netdb.h>
 #endif
 #endif
+#include <ctype.h>
 
 #include "response.h"
 #include "const.h"
@@ -173,6 +175,7 @@ struct str_option options_set[] =
 #ifdef MAPPING
 	{"UserMapFile", OP_STRING, &sp_mapfile, PATH_MAX, NULL},
 	{"DoMapping", OP_BOOLEAN, &domapping, 0, NULL},
+	{"RequiredMapping", OP_BOOLEAN, &reqmapping, 0, NULL},
 #endif
 #ifdef NONIPVIRTUALS
 	{"AllowNonIP", OP_BOOLEAN, &allownonip, 0, NULL},
@@ -180,6 +183,10 @@ struct str_option options_set[] =
 #ifdef CREATEMAILDROP
 	{"CreateMailDrop", OP_BOOLEAN, &createmaildrop, 0, NULL},
 #endif
+#ifdef STATISTICS
+	{"LogStatistics", OP_BOOLEAN, &logstatistics, 0, NULL},
+#endif
+	{"LogPriority", OP_STRING, &logpriority, 64, check_logpriority},
 	{NULL, 0, NULL, 0, NULL}
 };
 
@@ -201,7 +208,7 @@ char maildrop_name[PATH_MAX];
 char maildrop_type[MAXMDTYPENAMELENGTH];
 #ifdef MAPPING
 char sp_mapfile[PATH_MAX];
-int domapping = 0;
+int domapping = 0, reqmapping = 1;
 char mapusername[MAXARGLN + 1];
 #endif
 #ifdef APOP
@@ -236,22 +243,24 @@ int createmaildrop = 0;
 #if defined(LOG_EXTEND) || defined(LOG_CONNECT)
 char ahname[384];
 #endif
-
+#ifdef STATISTICS
+int logstatistics = 1;
+#endif
 
 void check_wccount(void) {
 	if (wccount != 0)
 		if ((--wccount) == 0) {
 #ifdef LOG_EXTEND
-			pop_log(POP_PRIORITY, "wrong commands limit exceeded - %.384s", ahname);
+			pop_log(pop_priority, "wrong commands limit exceeded - %.384s", ahname);
 #else
-			pop_log(POP_PRIORITY, "wrong commands limit exceeded");
+			pop_log(pop_priority, "wrong commands limit exceeded");
 #endif
 			exit(1);
 		};
-};
+}
 
 int expand_dir(char *dir, char *homedir) {
-	unsigned char filename[PATH_MAX], *tmp, *tmp2;
+	char filename[PATH_MAX], *tmp, *tmp2;
 	int breakwhile = 0;
 	
 	filename[0] = 0;
@@ -315,7 +324,7 @@ int expand_dir(char *dir, char *homedir) {
 				breakwhile = 0;
 				tmp = tmp2 + 3;
 				break;
-			default:
+			default: ;
 		};
 		if (breakwhile)
 			break;
@@ -333,7 +342,7 @@ int expand_dir(char *dir, char *homedir) {
 		return -1;
 	strcat(dir, filename);
 	return 0;
-};
+}
 
 #ifdef BULLETINS
 void add_bulletins(char *homedir) {
@@ -355,7 +364,7 @@ void add_bulletins(char *homedir) {
 		return;
 	};
 	if ((strlen(bulldir) + 1) >= sizeof(bullmsg)) {
-		pop_log(POP_PRIORITY, "unexpected error - contact author of program");
+		pop_log(pop_priority, "unexpected error - contact author of program");
 		send_error("fatal error");
 		exit(1);
 	};
@@ -363,34 +372,41 @@ void add_bulletins(char *homedir) {
 	strcat(bullmsg, "/");
 	tmp = bullmsg + strlen(bullmsg);
 	if (expand_dir(userbullfile, homedir) < 0) {
-		pop_log(POP_PRIORITY, "file name too long");
+		pop_log(pop_priority, "file name too long");
 		send_error("fatal error");
 		exit(1);
 	};
-	if ((fd = open(userbullfile, O_RDONLY | O_CREAT, 0600)) < 0) {
+	modtime = 1;
+	fd = open(userbullfile, O_RDONLY);
+	if ((fd < 0) && (errno == ENOENT)) {
+		fd = open(userbullfile, O_RDONLY | O_CREAT | O_EXCL, 0600);
+		modtime = 0;
+	};
+	if (fd < 0) {
 	/* It runs with user privileges, so users can do what they want */
-		pop_log(POP_PRIORITY, "can't open or create file: %.1024s", userbullfile);
+		pop_log(pop_priority, "can't open or create file: %.1024s", userbullfile);
 		pop_error("open");
 		send_error("fatal error");
 		exit(1);
 	};
 	if (fstat(fd, &stbuf) < 0) {
 		close(fd);
-		pop_log(POP_PRIORITY, "can't stat file: %.1024s", userbullfile);
+		pop_log(pop_priority, "can't stat file: %.1024s", userbullfile);
 		pop_error("stat");
 		send_error("fatal error");
 		exit(1);
 	};
-	modtime = stbuf.st_mtime;
+	if (modtime)
+		modtime = stbuf.st_mtime;
 	actime = stbuf.st_atime;
 	close(fd); /* We have modification time of user bulletin file */
 	if (!S_ISREG(stbuf.st_mode) || (stbuf.st_mode & 2) || (getuid() != stbuf.st_uid)) {
-		pop_log(POP_PRIORITY, "user bulletin file has wrong mode");
+		pop_log(pop_priority, "user bulletin file has wrong mode");
 		send_error("fatal error");
 		exit(1);
 	};
 	if ((dirstream = opendir(bulldir)) == NULL) {
-		pop_log(POP_PRIORITY, "opendir failed: %.1024s", bulldir);
+		pop_log(pop_priority, "opendir failed: %.1024s", bulldir);
 		pop_error("opendir");
 		send_error("fatal error");
 		exit(1);
@@ -398,14 +414,14 @@ void add_bulletins(char *homedir) {
 	while ((dentry = readdir(dirstream)) != NULL) {
 		if ((NAMLEN(dentry) + strlen(bullmsg) + 1) > sizeof(bullmsg)) {
 			closedir(dirstream);
-			pop_log(POP_PRIORITY, "file name too long");			
+			pop_log(pop_priority, "file name too long");			
 			send_error("fatal error");
 			exit(1);
 		};
 		strcpy(tmp, dentry->d_name);
 		if (stat(bullmsg, &stbuf) < 0) {
 			closedir(dirstream);
-			pop_log(POP_PRIORITY, "stat failed: %.1024s", bullmsg);
+			pop_log(pop_priority, "stat failed: %.1024s", bullmsg);
 			pop_error("stat");
 			send_error("fatal error");
 			exit(1);
@@ -414,7 +430,7 @@ void add_bulletins(char *homedir) {
 			continue;
 		if ((fd = open(bullmsg, O_RDONLY)) < 0) {
 			closedir(dirstream);
-			pop_log(POP_PRIORITY, "can't open file: %.1024s", bullmsg);
+			pop_log(pop_priority, "can't open file: %.1024s", bullmsg);
 			pop_error("open");
 			send_error("fatal error");
 			exit(1);
@@ -423,7 +439,10 @@ void add_bulletins(char *homedir) {
 			close(fd);
 			continue;
 		};
-		maildrop->md_add_message(fd);
+		if (maildrop->md_add_message(fd) == 0) {
+			close(fd);
+			break;
+		};
 		close(fd);
 	};
 	closedir(dirstream);
@@ -432,7 +451,7 @@ void add_bulletins(char *homedir) {
 	newmtime.actime = actime;
 	newmtime.modtime = time(NULL);
 	if (utime(userbullfile, &newmtime) < 0) {
-		pop_log(POP_PRIORITY, "can't set modification time of the file: %.1024s", userbullfile);
+		pop_log(pop_priority, "can't set modification time of the file: %.1024s", userbullfile);
 		pop_error("utime");
 #else
 	newmtime[0].tv_usec = newmtime[1].tv_usec = 0;
@@ -444,17 +463,20 @@ void add_bulletins(char *homedir) {
 		send_error("fatal error");
 		exit(1);
 	};
-};
+}
 #endif
 
 
 void at_transaction_end(void)
 {
 	maildrop->md_release();
-};
+}
 
 void end_trans_state(char *arg)
 {
+#ifdef STATISTICS
+	int deleted = 0, deletedsize = 0, left = 0, leftsize = 0;
+#endif
 #ifdef EXPIRATION
 	int tmp;
 	struct expiration *curexp;
@@ -467,50 +489,67 @@ void end_trans_state(char *arg)
 			curexp = &unrexp;
 		if (curexp->enabled == 0)
 			continue;
-		if (messages[tmp].msg_time > acttime) /* Clock skew??? */
+		if ((messages[tmp].msg_time == -1) ||
+		    (messages[tmp].msg_time > acttime)) /* Clock skew??? */
 			continue;
 		if ((acttime - messages[tmp].msg_time) >= curexp->expperiod)
 			messages[tmp].deleted = 1;
 	};
-#endif
 	maildrop->md_update();
+#endif /* EXPIRATION */
+
+#ifdef STATISTICS
+	for (tmp = 0; tmp < msgnr; tmp++)
+		if (messages[tmp].deleted) {
+			deleted++;
+			deletedsize += messages[tmp].crlfsize;
+		} else {
+			left++;
+			leftsize += messages[tmp].crlfsize;
+		}
+	if (logstatistics)
+		pop_log(pop_priority, "Stats: %.40s %d %d %d %d", username, deleted, deletedsize, \
+			left, leftsize);
+#endif
 #ifdef MAPPING
 	if (domapping)
 #ifdef LOG_EXTEND
-		pop_log(POP_PRIORITY, "session ended for mapped user %.40s - %.384s", username, ahname);
+		pop_log(pop_priority, "session ended for mapped user %.40s - %.384s", username, ahname);
 #else
-		pop_log(POP_PRIORITY, "session ended for mapped user %.40s", username);
+		pop_log(pop_priority, "session ended for mapped user %.40s", username);
 #endif
 	else
 #ifdef LOG_EXTEND
-		pop_log(POP_PRIORITY, "session ended for user %.40s - %.384s", username, ahname);
+		pop_log(pop_priority, "session ended for user %.40s - %.384s", username, ahname);
 #else
-		pop_log(POP_PRIORITY, "session ended for user %.40s", username);
+		pop_log(pop_priority, "session ended for user %.40s", username);
 #endif
 #else /* MAPPING */
 #ifdef LOG_EXTEND
-	pop_log(POP_PRIORITY, "session ended for user %.40s - %.384s", username, ahname);
+	pop_log(pop_priority, "session ended for user %.40s - %.384s", username, ahname);
 #else
-	pop_log(POP_PRIORITY, "session ended for user %.40s", username);
+	pop_log(pop_priority, "session ended for user %.40s", username);
 #endif
 #endif /* MAPPING */
 
 	send_ok("session ended");
 	exit(0);
-};
+}
 
 
 void end_auth_state(char *arg) {
 #ifdef LOG_EXTEND
-	pop_log(POP_PRIORITY, "session ended - %.384s", ahname);
+	pop_log(pop_priority, "session ended - %.384s", ahname);
 #else
-	pop_log(POP_PRIORITY, "session ended");
+	pop_log(pop_priority, "session ended");
 #endif
 	send_ok("session ended");
 	exit(0);
-};
+}
 
 void get_username(char *arg) {
+	int tmp = 0;
+
 	if (strlen(arg) >= sizeof(username)) {
 		send_error("username too long");
 		check_wccount();
@@ -522,23 +561,27 @@ void get_username(char *arg) {
 			if (username[0] == 0) {
 				strncpy(username, arg, sizeof(username));
 				/* Pad with NULs, strlen(arg) < sizeof(username) */
+				while (username[tmp] != 0) {
+					username[tmp] = tolower(username[tmp]);
+					tmp++;
+				};
 				send_ok("username accepted");
 			} else {
 				send_error("username already set");
 				check_wccount();
 			};
-};
+}
 
 #ifdef LASTCMD
 void last_hack(char *arg) {
 	send_ok("0");
-};
+}
 #endif
 
 void do_stat(char *arg)
 {
 	md_stat();
-};
+}
 
 void do_list(char *arg)
 {
@@ -552,7 +595,7 @@ void do_list(char *arg)
 			md_list(number);
 	} else
 		md_list(0);
-};
+}
 
 void do_retrieve(char *arg)
 {
@@ -568,7 +611,7 @@ void do_retrieve(char *arg)
 		check_wccount();
 	} else
 		maildrop->md_retrieve(number);
-};
+}
 
 void do_delete(char *arg)
 {
@@ -584,12 +627,12 @@ void do_delete(char *arg)
 		check_wccount();
 	} else
 		md_delete(number);
-};
+}
 
 void do_reset(char *arg)
 {
 	md_reset();
-};
+}
 
 void do_top(char *arg)
 {
@@ -606,7 +649,7 @@ void do_top(char *arg)
 		return;
 	};
 	maildrop->md_top(number, lcount);
-};
+}
 
 void do_uidl(char *arg)
 {
@@ -627,7 +670,7 @@ void do_uidl(char *arg)
 			md_uidl(number, maildrop->md_md5_uidl_message);
 	} else
 		md_uidl(0, maildrop->md_md5_uidl_message);
-};
+}
 
 
 char *findcrlf(char *where, size_t length)
@@ -642,7 +685,7 @@ char *findcrlf(char *where, size_t length)
 		if ((where[tmp] == '\r') && (where[tmp + 1] == '\n'))
 			return &where[tmp];
 	return NULL;
-};
+}
 
 
 void read_command(char *cmd)
@@ -698,23 +741,25 @@ void read_command(char *cmd)
 		break;
 	};
 	alarm(0);
-};
+}
 
 void sig_handler(int number) {
 	if (number == SIGALRM)
 #ifdef LOG_EXTEND
-		pop_log(POP_PRIORITY, "autologout time elapsed - %.384s", ahname);
+		pop_log(pop_priority, "autologout time elapsed - %.384s", ahname);
 #else
-		pop_log(POP_PRIORITY, "autologout time elapsed");
+		pop_log(pop_priority, "autologout time elapsed");
 #endif
 	exit(1);
-};
+}
 
 ssize_t write_loop(int fd, void *abuf, size_t acount) {
 	char *tmpbuf = (char *) abuf;
 	size_t writed = 0;
 	ssize_t tmp;
 	
+	if (acount == 0)
+		return 0;
 	while ((tmp = write(fd, tmpbuf + writed, acount - writed)) > 0) {
 		writed += tmp;
 		if (writed == acount)
@@ -723,7 +768,7 @@ ssize_t write_loop(int fd, void *abuf, size_t acount) {
 	if (tmp < 0)
 		return -1;
 	return writed;
-};
+}
 
 int write_string(int fd, char *abuf) {
 	size_t tmp = strlen(abuf);
@@ -733,7 +778,7 @@ int write_string(int fd, char *abuf) {
 	if (write_loop(fd, abuf, tmp) != tmp)
 		return -1;
 	return 0;
-};
+}
 
 void do_authentication(char *pass) {
 #ifdef APOP
@@ -750,7 +795,7 @@ void do_authentication(char *pass) {
 	if (write_loop(tunnel[1], &tmp, sizeof(tmp)) != sizeof(tmp)) {
 		memset(pass, 0, strlen(pass));
 		close(tunnel[1]);
-		pop_log(POP_PRIORITY, "can't write to pipe");
+		pop_log(pop_priority, "can't write to pipe");
 		pop_error("write");
 		send_error("fatal error");
 		exit(1);
@@ -766,7 +811,7 @@ void do_authentication(char *pass) {
 	if (write_string(tunnel[1], username) < 0) {
 		memset(pass, 0, strlen(pass));
 		close(tunnel[1]);
-		pop_log(POP_PRIORITY, "can't write to pipe");
+		pop_log(pop_priority, "can't write to pipe");
 		pop_error("write");
 		send_error("fatal error");
 		exit(1);
@@ -774,7 +819,7 @@ void do_authentication(char *pass) {
 	if (write_string(tunnel[1], pass) < 0) {
 		memset(pass, 0, strlen(pass));
 		close(tunnel[1]);	
-		pop_log(POP_PRIORITY, "can't write to pipe");
+		pop_log(pop_priority, "can't write to pipe");
 		pop_error("write");
 		send_error("fatal error");
 		exit(1);
@@ -782,7 +827,7 @@ void do_authentication(char *pass) {
 	memset(pass, 0, strlen(pass));
 	auth_finished = 1;
 	return;
-};
+}
 
 #ifdef APOP
 void do_apop(char *arg) {
@@ -830,7 +875,7 @@ void do_apop(char *arg) {
 	if (write_loop(tunnel[1], &tmp2, sizeof(tmp2)) != sizeof(tmp2)) {
 		close(tunnel[1]);
 		memset(tmp, 0, strlen(tmp));
-		pop_log(POP_PRIORITY, "can't write to pipe");
+		pop_log(pop_priority, "can't write to pipe");
 		pop_error("write");
 		send_error("fatal error");
 		exit(1);
@@ -838,7 +883,7 @@ void do_apop(char *arg) {
 	if (write_string(tunnel[1], apoptimestamp) < 0) {
 		close(tunnel[1]);
 		memset(tmp, 0, strlen(tmp));
-		pop_log(POP_PRIORITY, "can't write to pipe");
+		pop_log(pop_priority, "can't write to pipe");
 		pop_error("write");
 		send_error("fatal error");
 		exit(1);
@@ -853,7 +898,7 @@ void do_apop(char *arg) {
 	if (write_string(tunnel[1], username) < 0) {
 		close(tunnel[1]);
 		memset(tmp, 0, strlen(tmp));
-		pop_log(POP_PRIORITY, "can't write to pipe");
+		pop_log(pop_priority, "can't write to pipe");
 		pop_error("write");
 		send_error("fatal error");
 		exit(1);
@@ -861,7 +906,7 @@ void do_apop(char *arg) {
 	if (write_string(tunnel[1], tmp) < 0) {
 		close(tunnel[1]);
 		memset(tmp, 0, strlen(tmp));
-		pop_log(POP_PRIORITY, "can't write to pipe");
+		pop_log(pop_priority, "can't write to pipe");
 		pop_error("write");
 		send_error("fatal error");
 		exit(1);
@@ -869,7 +914,7 @@ void do_apop(char *arg) {
 	memset(tmp, 0, strlen(tmp));
 	auth_finished = 1;
 	return;
-};
+}
 #endif /* APOP */
 
 ssize_t read_loop(int fd, void *abuf, size_t acount) {
@@ -877,6 +922,8 @@ ssize_t read_loop(int fd, void *abuf, size_t acount) {
 	char *tmpbuf = (char *) abuf;
 	ssize_t tmp, cread = 0;
 	
+	if (acount == 0)
+		return 0;
 	while ((tmp = read(fd, tmpbuf + cread, acount - cread)) > 0) {
 		cread += tmp;
 		if (cread == acount)
@@ -885,16 +932,16 @@ ssize_t read_loop(int fd, void *abuf, size_t acount) {
 	if (tmp < 0)
 		return -1;
 	return cread;
-};
+}
 
 void child_corrupted(void) {
 /* This function works with root privileges */
 	memset(password, 0, sizeof(password));
 	close(tunnel[0]);
-	pop_log(POP_PRIORITY, "child process corrupted");
+	pop_log(pop_priority, "child process corrupted");
 	wait(NULL);
 	exit(1);
-};
+}
 
 int read_string(int fd, char *abuf, size_t acount) {
 /* This function works with root privileges */
@@ -902,7 +949,8 @@ int read_string(int fd, char *abuf, size_t acount) {
 	
 	if (read_loop(fd, &tmp2, sizeof(tmp2)) != sizeof(tmp2))
 		return -1;		
-	if (tmp2 >= acount)
+	if ((tmp2 >= acount) || (tmp2 < 0)) /* tmp2 is an unsigned number,
+	so there wasn't security hole, however "szczezonego Pan Bog szczeze" */
 		return -1;
 	if (read_loop(fd, abuf, tmp2) != tmp2)
 		return -1;
@@ -910,7 +958,7 @@ int read_string(int fd, char *abuf, size_t acount) {
 	if (strlen(abuf) != tmp2)
 		return -1;
 	return 0;
-};
+}
 
 int set_privileges(uid_t auid, gid_t agid) {
 /* This function works with root privileges */
@@ -932,7 +980,7 @@ int set_privileges(uid_t auid, gid_t agid) {
 		return -1;
 	};
 	return 0; /* I see no reason why this function could fail */
-};
+}
 
 #ifdef STANDALONE
 int do_session(int argc, char **argv)
@@ -958,10 +1006,10 @@ int main(int argc, char **argv)
 #if  defined(LOG_EXTEND) || defined(LOG_CONNECT)
 #ifdef RESOLVE_HOSTNAME
 	struct hostent *hentname;
+#endif
 #ifdef SPIPV6
 	char ntopbuff[100];
 #endif
-#endif
 #endif
 
 
@@ -1011,20 +1059,30 @@ int main(int argc, char **argv)
 #else
 	if (peeraddr.sin_family != AF_INET) {
 #endif
-		pop_log(POP_PRIORITY, "peer address is not an IP address");
+		pop_log(pop_priority, "peer address is not an IP address");
 		exit(1);
 	};
 #if defined(LOG_EXTEND) || defined(LOG_CONNECT)
 #ifdef RESOLVE_HOSTNAME
-#ifdef SPIPV6			
-	if ((hentname = gethostbyaddr((char *)&peeraddr.saddr_in6.sin6_addr, addrln, peeraddr.saddr_in6.sin6_family)) != NULL)
-		snprintf(ahname, sizeof(ahname), "%.256s (%.100s)", hentname->h_name,
-		    (strncmp(inet_ntop(AF_INET6, &peeraddr.saddr_in6.sin6_addr, ntopbuff, \
-		    sizeof(ntopbuff)), "::ffff:", 7) == 0) ? ntopbuff + 7 : ntopbuff);
-	else
-		snprintf(ahname, sizeof(ahname), "%.100s",
-		    (strncmp(inet_ntop(AF_INET6, &peeraddr.saddr_in6.sin6_addr, ntopbuff, \
-		    sizeof(ntopbuff)), "::ffff:", 7) == 0) ? ntopbuff + 7 : ntopbuff);
+#ifdef SPIPV6
+	if (peeraddr.saddr_in6.sin6_family == AF_INET6) {
+		if ((hentname = gethostbyaddr((char *)&peeraddr.saddr_in6.sin6_addr, addrln, AF_INET6)) != NULL)
+			snprintf(ahname, sizeof(ahname), "%.256s (%.100s)", hentname->h_name,
+			    (strncmp(inet_ntop(AF_INET6, &peeraddr.saddr_in6.sin6_addr, ntopbuff, \
+			    sizeof(ntopbuff)), "::ffff:", 7) == 0) ? ntopbuff + 7 : ntopbuff);
+		else
+			snprintf(ahname, sizeof(ahname), "%.100s",
+			    (strncmp(inet_ntop(AF_INET6, &peeraddr.saddr_in6.sin6_addr, ntopbuff, \
+			    sizeof(ntopbuff)), "::ffff:", 7) == 0) ? ntopbuff + 7 : ntopbuff);
+	} else {
+		if ((hentname = gethostbyaddr((char *)&peeraddr.saddr_in.sin_addr.s_addr, sizeof(peeraddr.saddr_in.sin_addr.s_addr), AF_INET)) != NULL)
+			snprintf(ahname, sizeof(ahname), "%.256s (%.100s)", hentname->h_name,
+			    inet_ntoa(peeraddr.saddr_in.sin_addr));
+		else
+			snprintf(ahname, sizeof(ahname), "%.100s",
+			    inet_ntoa(peeraddr.saddr_in.sin_addr));
+
+	};
 #else
 	if ((hentname = gethostbyaddr((char *)&peeraddr.sin_addr.s_addr, sizeof(peeraddr.sin_addr.s_addr), AF_INET)) != NULL)
 		snprintf(ahname, sizeof(ahname), "%.256s (%.100s)", hentname->h_name,
@@ -1035,9 +1093,13 @@ int main(int argc, char **argv)
 #endif
 #else /* RESOLVE_HOSTNAME */
 #ifdef SPIPV6
-	snprintf(ahname, sizeof(ahname), "%.100s",
-	    (strncmp(inet_ntop(AF_INET6, &peeraddr.saddr_in6.sin6_addr, ntopbuff, \
-	    sizeof(ntopbuff)), "::ffff:", 7) == 0) ? ntopbuff + 7 : ntopbuff);
+	if (peeraddr.saddr_in6.sin6_family == AF_INET6)
+		snprintf(ahname, sizeof(ahname), "%.100s",
+		    (strncmp(inet_ntop(AF_INET6, &peeraddr.saddr_in6.sin6_addr, ntopbuff, \
+		    sizeof(ntopbuff)), "::ffff:", 7) == 0) ? ntopbuff + 7 : ntopbuff);
+	else
+		snprintf(ahname, sizeof(ahname), "%.100s",
+		    inet_ntoa(peeraddr.saddr_in.sin_addr));
 #else
 	snprintf(ahname, sizeof(ahname), "%.100s",
 	    inet_ntoa(peeraddr.sin_addr));
@@ -1045,7 +1107,7 @@ int main(int argc, char **argv)
 #endif /* RESOLVE_HOSTNAME */
 #ifdef LOG_CONNECT
 #ifndef STANDALONE
-	pop_log(POP_PRIORITY, "connect from %.384s", ahname);
+	pop_log(pop_priority, "connect from %.384s", ahname);
 #endif
 #endif
 #endif /* LOG_EXTEND || LOG_CONNECT */
@@ -1061,7 +1123,7 @@ int main(int argc, char **argv)
 		case 0:
 			close(tunnel[0]);
 			if ((spop3d = getpwnam(POPUSER)) == NULL) {
-				pop_log(POP_PRIORITY, "user %.40s not found", POPUSER);
+				pop_log(pop_priority, "user %.40s not found", POPUSER);
 				pop_error("getpwnam");
 				exit(1);
 			};
@@ -1110,7 +1172,7 @@ int main(int argc, char **argv)
 #ifdef NONIPVIRTUALS /* reread configuration file if needed */			
 			if (tmpvname != NULL) {
 				if (!allownonip)
-					pop_log(POP_PRIORITY, "unallowed non-IP based virtual hosting request rejected");
+					pop_log(pop_priority, "unallowed non-IP based virtual hosting request rejected");
 				else
 					if (parse_options(argc, argv, tmpvname + 1) < 0) {
 						send_error("fatal error");
@@ -1121,21 +1183,26 @@ int main(int argc, char **argv)
 #ifdef MAPPING
 			mapusername[0] = 0;
 			if (domapping) {
-				if (map_finduser(sp_mapfile, username, mapusername) < 0) {
+/* map_finduser() doesn't overflow mapusername - sizeof(mapusername) = MAXARGLN + 1 */
+				if (((tmp = map_finduser(sp_mapfile, username, mapusername)) < 0) && \
+				    (reqmapping)) {
 #ifdef LOG_EXTEND
-					pop_log(POP_PRIORITY, "authentication failed: can't map user name: %.40s - %.384s", username, ahname);
+					pop_log(pop_priority, "authentication failed: can't map user name: %.40s - %.384s", username, ahname);
 #else
-					pop_log(POP_PRIORITY, "authentication failed: can't map user name: %.40s", username);
+					pop_log(pop_priority, "authentication failed: can't map user name: %.40s", username);
 #endif
 					sleep(1);
 					send_error("authentication failed");
 					exit(1);
 				};
-				if ((strlen(mapusername) + 1)> sizeof(mapusername)) {
-					pop_log(POP_PRIORITY, "map: unexpected error - contact author of program!!!");
-					send_error("fatal error");
-					exit(1);
-				};
+				if (tmp < 0) /* && !reqmapping */
+					domapping = 0;
+				else
+					if ((strlen(mapusername) + 1) > sizeof(mapusername)) {
+						pop_log(pop_priority, "map: unexpected error - contact author of program!!!");
+						send_error("fatal error");
+						exit(1);
+					};
 			};
 			if (write_string(tunnel[1], mapusername) < 0) {
 				close(tunnel[1]);
@@ -1246,6 +1313,7 @@ int main(int argc, char **argv)
 			};
 			close(tunnel[0]);
 			wait(NULL);
+			check_logpriority(logpriority);
 #ifdef MAPPING
 			if (domapping)
 				userentry = getpwnam(mapusername);
@@ -1259,21 +1327,21 @@ int main(int argc, char **argv)
 #ifdef MAPPING
 				if (domapping)
 #ifdef LOG_EXTEND
-					pop_log(POP_PRIORITY, "authentication failed: no such user: %.40s - %.384s", mapusername, ahname);
+					pop_log(pop_priority, "authentication failed: no such user: %.40s - %.384s", mapusername, ahname);
 #else
-					pop_log(POP_PRIORITY, "authentication failed: no such user: %.40s", mapusername);
+					pop_log(pop_priority, "authentication failed: no such user: %.40s", mapusername);
 #endif
 				else
 #ifdef LOG_EXTEND
-					pop_log(POP_PRIORITY, "authentication failed: no such user: %.40s - %.384s", username, ahname);
+					pop_log(pop_priority, "authentication failed: no such user: %.40s - %.384s", username, ahname);
 #else
-					pop_log(POP_PRIORITY, "authentication failed: no such user: %.40s", username);				
+					pop_log(pop_priority, "authentication failed: no such user: %.40s", username);				
 #endif
 #else /* MAPPING */
 #ifdef LOG_EXTEND
-				pop_log(POP_PRIORITY, "authentication failed: no such user: %.40s - %.384s", username, ahname);
+				pop_log(pop_priority, "authentication failed: no such user: %.40s - %.384s", username, ahname);
 #else
-				pop_log(POP_PRIORITY, "authentication failed: no such user: %.40s", username);				
+				pop_log(pop_priority, "authentication failed: no such user: %.40s", username);				
 #endif
 #endif /* MAPPING */
 				sleep(1);
@@ -1296,21 +1364,21 @@ int main(int argc, char **argv)
 #ifdef MAPPING
 				if (domapping)
 #ifdef LOG_EXTEND
-					pop_log(POP_PRIORITY, "authentication failed for mapped user %.40s - %.384s", username, ahname);
+					pop_log(pop_priority, "authentication failed for mapped user %.40s - %.384s", username, ahname);
 #else
-					pop_log(POP_PRIORITY, "authentication failed for mapped user %.40s", username);
+					pop_log(pop_priority, "authentication failed for mapped user %.40s", username);
 #endif
 				else
 #ifdef LOG_EXTEND
-					pop_log(POP_PRIORITY, "authentication failed for user %.40s - %.384s", username, ahname);
+					pop_log(pop_priority, "authentication failed for user %.40s - %.384s", username, ahname);
 #else
-					pop_log(POP_PRIORITY, "authentication failed for user %.40s", username);
+					pop_log(pop_priority, "authentication failed for user %.40s", username);
 #endif
 #else /* MAPPING */
 #ifdef LOG_EXTEND
-				pop_log(POP_PRIORITY, "authentication failed for user %.40s - %.384s", username, ahname);
+				pop_log(pop_priority, "authentication failed for user %.40s - %.384s", username, ahname);
 #else
-				pop_log(POP_PRIORITY, "authentication failed for user %.40s", username);
+				pop_log(pop_priority, "authentication failed for user %.40s", username);
 #endif
 #endif /* MAPPING */
 				sleep(1);
@@ -1320,21 +1388,21 @@ int main(int argc, char **argv)
 #ifdef MAPPING
 			if (domapping)
 #ifdef LOG_EXTEND
-				pop_log(POP_PRIORITY, "mapped user %.40s authenticated - %.384s", username, ahname);
+				pop_log(pop_priority, "mapped user %.40s authenticated - %.384s", username, ahname);
 #else
-				pop_log(POP_PRIORITY, "mapped user %.40s authenticated", username);
+				pop_log(pop_priority, "mapped user %.40s authenticated", username);
 #endif
 			else
 #ifdef LOG_EXTEND
-				pop_log(POP_PRIORITY, "user %.40s authenticated - %.384s", username, ahname);
+				pop_log(pop_priority, "user %.40s authenticated - %.384s", username, ahname);
 #else
-				pop_log(POP_PRIORITY, "user %.40s authenticated", username);
+				pop_log(pop_priority, "user %.40s authenticated", username);
 #endif
 #else /* MAPPING */
 #ifdef LOG_EXTEND
-			pop_log(POP_PRIORITY, "user %.40s authenticated - %.384s", username, ahname);
+			pop_log(pop_priority, "user %.40s authenticated - %.384s", username, ahname);
 #else
-			pop_log(POP_PRIORITY, "user %.40s authenticated", username);
+			pop_log(pop_priority, "user %.40s authenticated", username);
 #endif
 
 #endif /* MAPPING */
@@ -1348,9 +1416,9 @@ int main(int argc, char **argv)
 #endif
 				memset(password, 0, strlen(password));
 #ifdef LOG_EXTEND
-				pop_log(POP_PRIORITY, "root login not allowed - %.384s", ahname);
+				pop_log(pop_priority, "root login not allowed - %.384s", ahname);
 #else
-				pop_log(POP_PRIORITY, "root login not allowed");
+				pop_log(pop_priority, "root login not allowed");
 #endif
 				sleep(1);
 				send_error("authentication failed");
@@ -1375,9 +1443,9 @@ int main(int argc, char **argv)
 				if (apop_secret[0] == 0) {
 					memset(password, 0, strlen(password));
 #ifdef LOG_EXTEND
-					pop_log(POP_PRIORITY, "can't find APOP secret for user %.40s - %.384s", username, ahname);
+					pop_log(pop_priority, "can't find APOP secret for user %.40s - %.384s", username, ahname);
 #else
-					pop_log(POP_PRIORITY, "can't find APOP secret for user %.40s", username);
+					pop_log(pop_priority, "can't find APOP secret for user %.40s", username);
 #endif
 					sleep(1);
 					send_error("authentication failed");
@@ -1387,25 +1455,25 @@ int main(int argc, char **argv)
 					memset(password, 0, strlen(password));
 					memset(apop_secret, 0, strlen(apop_secret));
 #ifdef LOG_EXTEND
-					pop_log(POP_PRIORITY, "APOP authentication failed for user %.40s - %.384s", username, ahname);
+					pop_log(pop_priority, "APOP authentication failed for user %.40s - %.384s", username, ahname);
 #else
-					pop_log(POP_PRIORITY, "APOP authentication failed for user %.40s", username);
+					pop_log(pop_priority, "APOP authentication failed for user %.40s", username);
 #endif
 					sleep(1);
 					send_error("authentication failed");
 					exit(1);
 				};
 #ifdef LOG_EXTEND
-				pop_log(POP_PRIORITY, "user %.40s authenticated through APOP - %.384s", username, ahname);
+				pop_log(pop_priority, "user %.40s authenticated through APOP - %.384s", username, ahname);
 #else
-				pop_log(POP_PRIORITY, "user %.40s authenticated through APOP", username);
+				pop_log(pop_priority, "user %.40s authenticated through APOP", username);
 #endif
 			};
 			memset(apop_secret, 0, sizeof(apop_secret));
 #endif
 			memset(password, 0, strlen(password));
 			if (expand_dir(maildrop_name, userentry->pw_dir) < 0) {
-				pop_log(POP_PRIORITY, "maildrop name too long");
+				pop_log(pop_priority, "maildrop name too long");
 				sleep(1);
 				send_error("authentication failed");
 				exit(1);
@@ -1413,7 +1481,7 @@ int main(int argc, char **argv)
 			maildrop = find_maildrop(maildrop_type);
 			maildrop->md_init();
 			if (atexit(at_transaction_end) < 0) {
-				pop_log(POP_PRIORITY, "weird error");
+				pop_log(pop_priority, "weird error");
 				send_error("fatal error");
 				exit(1);
 			};
@@ -1435,4 +1503,4 @@ int main(int argc, char **argv)
 					act_cmd->handler(cmd + strlen(act_cmd->name) + 1);
 			};
 	};
-};
+}
